/* AutoGuider  V1.00 */

#include <MsTimer2.h>
#include <Bounce.h>

#define LED   13

#define CLK_M1     5
#define RESET_M1   3
#define ENABLE_M1  4
#define CWCCW_M1  19
#define TQ1_M1     6
#define TQ2_M1     7

#define CLK_M2    10
#define RESET_M2   8
#define ENABLE_M2  9
#define CWCCW_M2   2
#define TQ1_M2    11
#define TQ2_M2    12

#define SW1       14
#define SW2       15

#define SW_ON     LOW
#define SW_OFF    HIGH

#define MOTOR_STOP 0
#define MOTOR_RUN  1

#define CCW      HIGH
#define CW       LOW


#define COMMAND_NORTH       1
#define COMMAND_SOUTH       2
#define COMMAND_EAST        3
#define COMMAND_WEST        4
#define COMMAND_MOVE        10
#define COMMAND_MOVE_NORTH  11
#define COMMAND_MOVE_SOUTH  12
#define COMMAND_MOVE_EAST   13
#define COMMAND_MOVE_WEST   14
#define COMMAND_QUIT        20
#define COMMAND_QUIT_NORTH  21
#define COMMAND_QUIT_SOUTH  22
#define COMMAND_QUIT_EAST   23
#define COMMAND_QUIT_WEST   24
#define COMMAND_OTHER       0

/*
#define arm_length                250.0  // [mm]
#define initial_stride             80.0  // [mm]
#define step_rotation              1600  // [step/rotation]
#define screw_pitch                 2.0  // [mm]
#define angular_velocity    7.29212E-08  // [rad/msec]
*/

#define MAX_TORQUE 100
#define MIN_TORQUE 25

int motor1_stat = MOTOR_STOP;
int motor2_stat = MOTOR_STOP;
int motor_rotate = CCW;

/*
unsigned long time;
unsigned long start_time;
unsigned long ptime;
unsigned long ntime;
unsigned long div_time;
unsigned long step_count;
*/

// float mmstep;

Bounce bouncerSW1 = Bounce( SW1, 5 );
Bounce bouncerSW2 = Bounce( SW2, 5 );


void setup()   {                
  pinMode(LED, OUTPUT);  
  
  pinMode(CLK_M1,    OUTPUT);     
  pinMode(RESET_M1,  OUTPUT);     
  pinMode(ENABLE_M1, OUTPUT);     
  pinMode(CWCCW_M1,  OUTPUT);
  pinMode(TQ1_M1,    OUTPUT);
  pinMode(TQ2_M1,    OUTPUT);

  pinMode(CLK_M2,    OUTPUT);     
  pinMode(RESET_M2,  OUTPUT);     
  pinMode(ENABLE_M2, OUTPUT);     
  pinMode(CWCCW_M2,  OUTPUT);
  pinMode(TQ1_M2,    OUTPUT);
  pinMode(TQ2_M2,    OUTPUT);

  pinMode(SW1, INPUT);
  pinMode(SW2, INPUT);
  
  stop_motor( 1 );
  stop_motor( 2 );

  init_motor( 1 );
  init_motor( 2 );
  
  set_torque( MAX_TORQUE, 1 );
  set_torque( MAX_TORQUE, 2 );

  MsTimer2::set(1,motor_run);
  MsTimer2::start();
  
  Serial.begin(9600);
}

void motor_run()
{
  if( motor1_stat == MOTOR_RUN ) {
    digitalWrite(CLK_M1, HIGH);
  }
  if( motor2_stat == MOTOR_RUN ) {
    digitalWrite(CLK_M2, HIGH);
  }
  delayMicroseconds(10);
  digitalWrite(CLK_M1, LOW);
  digitalWrite(CLK_M2, LOW);
}

void loop()                     
{
  int command_no;
  
  command_no = get_command();
  exec_command( command_no );
}

void exec_command( int command_no )
{
  switch( command_no ) {
    case COMMAND_MOVE_NORTH:
      set_torque( MAX_TORQUE, 2 );
      digitalWrite(CWCCW_M2,CW );
      motor2_stat = MOTOR_RUN;
      break;
    case COMMAND_MOVE_SOUTH:
      set_torque( MAX_TORQUE, 2 );
      digitalWrite(CWCCW_M2,CCW);
      motor2_stat = MOTOR_RUN;
      break;
    case COMMAND_MOVE_EAST:
      set_torque( MAX_TORQUE, 1 );   
      digitalWrite(CWCCW_M1,CW );
      motor1_stat = MOTOR_RUN;
      break;
    case COMMAND_MOVE_WEST:
      set_torque( MAX_TORQUE, 1 );    
      digitalWrite(CWCCW_M1,CCW);
      motor1_stat = MOTOR_RUN;
      break;
    case COMMAND_QUIT_NORTH:
      set_torque( MIN_TORQUE, 2 );    
      motor2_stat = MOTOR_STOP;
      break;
    case COMMAND_QUIT_SOUTH:
      set_torque( MIN_TORQUE, 2 );    
      motor2_stat = MOTOR_STOP;
      break;
    case COMMAND_QUIT_EAST:
      set_torque( MIN_TORQUE, 1 );    
      motor1_stat = MOTOR_STOP;
      break;
    case COMMAND_QUIT_WEST:
      set_torque( MIN_TORQUE, 1 );    
      motor1_stat = MOTOR_STOP;
      break;
    case COMMAND_QUIT:
      motor1_stat = MOTOR_STOP;
      motor2_stat = MOTOR_STOP;
      break;
    default:
      digitalWrite( 13, HIGH );
      break;
  }
}

int get_command()
{
  char rbyte = 0;
  int command_no = 0;
  
  while(true) {
    if (Serial.available() > 0) {
      rbyte = Serial.read();
      if ( rbyte == '#' ) {
        break;
      }
      if ( rbyte == ':' ) {
        command_no = 0;
      }
      switch( rbyte ) {
        case 'M': command_no =  COMMAND_MOVE;  break;
        case 'Q': command_no =  COMMAND_QUIT;  break;
        case 'n': command_no += COMMAND_NORTH; break;
        case 's': command_no += COMMAND_SOUTH; break;
        case 'e': command_no += COMMAND_EAST;  break;
        case 'w': command_no += COMMAND_WEST;  break;
      }
    }
  }

  return command_no;
}  

void init_motor(int motor_No)
{
  switch( motor_No ) {
    case 1:
      digitalWrite( RESET_M1,  LOW );
      digitalWrite( ENABLE_M1, LOW );
      digitalWrite( CWCCW_M1,  CCW );
      delayMicroseconds(100);
      digitalWrite( ENABLE_M1, HIGH );
      delayMicroseconds(100);
      digitalWrite( RESET_M1,  HIGH );
      delayMicroseconds(100);
      break;
    case 2:
      digitalWrite( RESET_M2,  LOW );
      digitalWrite( ENABLE_M2, LOW );
      digitalWrite( CWCCW_M2,  CCW );
      delayMicroseconds(100);
      digitalWrite( ENABLE_M2, HIGH );
      delayMicroseconds(100);
      digitalWrite( RESET_M2,  HIGH );
      delayMicroseconds(100);
      break;
  }
}

void stop_motor( int motor_No )
{
  switch( motor_No ) {
    case 1:
      digitalWrite( RESET_M1,  LOW );
      digitalWrite( ENABLE_M1, LOW );
      break;
    case 2:
      digitalWrite( RESET_M2,  LOW );
      digitalWrite( ENABLE_M2, LOW );   
      break;
  }
}

void set_torque( int torq, int motor_no )
{
   switch( motor_no ) {
    case 1:
      switch( torq ) {
        case 100: digitalWrite( TQ1_M1, LOW  ); digitalWrite( TQ2_M1, LOW  ); break;
        case  75: digitalWrite( TQ1_M1, HIGH ); digitalWrite( TQ2_M1, LOW  ); break;
        case  50: digitalWrite( TQ1_M1, LOW  ); digitalWrite( TQ2_M1, HIGH ); break;
        case  25: digitalWrite( TQ1_M1, HIGH ); digitalWrite( TQ2_M1, HIGH ); break;
      }
      break;
    case 2:
      switch( torq ) {
        case 100: digitalWrite( TQ1_M2, LOW  ); digitalWrite( TQ2_M2, LOW  ); break;
        case  75: digitalWrite( TQ1_M2, HIGH ); digitalWrite( TQ2_M2, LOW  ); break;
        case  50: digitalWrite( TQ1_M2, LOW  ); digitalWrite( TQ2_M2, HIGH ); break;
        case  25: digitalWrite( TQ1_M2, HIGH ); digitalWrite( TQ2_M2, HIGH ); break;
      }
      break;
  } 
}
